pub mod logger;
pub mod udp;

use crate::medium::logger::Logger;
use crate::medium::udp::UdpMedium;
use log::info;
use serde::Deserialize;
use std::io::{Read, Write};
use std::net::SocketAddr;

pub trait ReadMedium: Read + Send {}

pub trait WriteMedium: Write + Send {}

pub trait Medium: Read + Write + Send {
    fn split(&self) -> (Box<dyn ReadMedium>, Box<dyn WriteMedium>);

    fn name(&self) -> String;

    fn init(&mut self) -> Result<(), std::io::Error> {
        info!("Initialized {} Medium", self.name());
        Ok(())
    }
}

#[derive(Debug, Deserialize)]
#[serde(tag = "medium")]
pub enum Mediums {
    Udp {
        local_addr: SocketAddr,
        remote_addr: SocketAddr,
    },
    Logger,
}

impl Mediums {
    pub fn create_medium(&self) -> Box<dyn Medium> {
        match self {
            Mediums::Udp {
                local_addr,
                remote_addr,
            } => Box::new(UdpMedium::new(*local_addr, *remote_addr)),
            Mediums::Logger => Box::new(Logger),
        }
    }
}
